# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.source_context_v1.types import source_context as source_context_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.protobuf import wrappers_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.devtools.clouddebugger.v2",
    manifest={
        "FormatMessage",
        "StatusMessage",
        "SourceLocation",
        "Variable",
        "StackFrame",
        "Breakpoint",
        "Debuggee",
    },
)


class FormatMessage(proto.Message):
    r"""Represents a message with parameters.

    Attributes:
        format_ (str):
            Format template for the message. The ``format`` uses
            placeholders ``$0``, ``$1``, etc. to reference parameters.
            ``$$`` can be used to denote the ``$`` character.

            Examples:

            -  ``Failed to load '$0' which helps debug $1 the first time it is loaded. Again, $0 is very important.``
            -  ``Please pay $$10 to use $0 instead of $1.``
        parameters (Sequence[str]):
            Optional parameters to be embedded into the
            message.
    """

    format_ = proto.Field(
        proto.STRING,
        number=1,
    )
    parameters = proto.RepeatedField(
        proto.STRING,
        number=2,
    )


class StatusMessage(proto.Message):
    r"""Represents a contextual status message. The message can indicate an
    error or informational status, and refer to specific parts of the
    containing object. For example, the ``Breakpoint.status`` field can
    indicate an error referring to the ``BREAKPOINT_SOURCE_LOCATION``
    with the message ``Location not found``.

    Attributes:
        is_error (bool):
            Distinguishes errors from informational
            messages.
        refers_to (google.cloud.debugger_v2.types.StatusMessage.Reference):
            Reference to which the message applies.
        description (google.cloud.debugger_v2.types.FormatMessage):
            Status message text.
    """

    class Reference(proto.Enum):
        r"""Enumerates references to which the message applies."""
        UNSPECIFIED = 0
        BREAKPOINT_SOURCE_LOCATION = 3
        BREAKPOINT_CONDITION = 4
        BREAKPOINT_EXPRESSION = 7
        BREAKPOINT_AGE = 8
        VARIABLE_NAME = 5
        VARIABLE_VALUE = 6

    is_error = proto.Field(
        proto.BOOL,
        number=1,
    )
    refers_to = proto.Field(
        proto.ENUM,
        number=2,
        enum=Reference,
    )
    description = proto.Field(
        proto.MESSAGE,
        number=3,
        message="FormatMessage",
    )


class SourceLocation(proto.Message):
    r"""Represents a location in the source code.

    Attributes:
        path (str):
            Path to the source file within the source
            context of the target binary.
        line (int):
            Line inside the file. The first line in the file has the
            value ``1``.
        column (int):
            Column within a line. The first column in a line as the
            value ``1``. Agents that do not support setting breakpoints
            on specific columns ignore this field.
    """

    path = proto.Field(
        proto.STRING,
        number=1,
    )
    line = proto.Field(
        proto.INT32,
        number=2,
    )
    column = proto.Field(
        proto.INT32,
        number=3,
    )


class Variable(proto.Message):
    r"""Represents a variable or an argument possibly of a compound object
    type. Note how the following variables are represented:

    1) A simple variable:

       int x = 5

       { name: "x", value: "5", type: "int" } // Captured variable

    2) A compound object:

       struct T { int m1; int m2; }; T x = { 3, 7 };

       { // Captured variable name: "x", type: "T", members { name:
       "m1", value: "3", type: "int" }, members { name: "m2", value:
       "7", type: "int" } }

    3) A pointer where the pointee was captured:

       T x = { 3, 7 }; T\* p = &x;

       { // Captured variable name: "p", type: "T*", value:
       "0x00500500", members { name: "m1", value: "3", type: "int" },
       members { name: "m2", value: "7", type: "int" } }

    4) A pointer where the pointee was not captured:

       T\* p = new T;

       { // Captured variable name: "p", type: "T*", value: "0x00400400"
       status { is_error: true, description { format: "unavailable" } }
       }

    The status should describe the reason for the missing value, such as
    ``<optimized out>``, ``<inaccessible>``,
    ``<pointers limit reached>``.

    Note that a null pointer should not have members.

    5) An unnamed value:

       int\* p = new int(7);

       { // Captured variable name: "p", value: "0x00500500", type:
       "int*", members { value: "7", type: "int" } }

    6) An unnamed pointer where the pointee was not captured:

       int\* p = new int(7); int*\* pp = &p;

       { // Captured variable name: "pp", value: "0x00500500", type:
       "int**", members { value: "0x00400400", type: "int*" status {
       is_error: true, description: { format: "unavailable" } } } } }

    To optimize computation, memory and network traffic, variables that
    repeat in the output multiple times can be stored once in a shared
    variable table and be referenced using the ``var_table_index``
    field. The variables stored in the shared table are nameless and are
    essentially a partition of the complete variable. To reconstruct the
    complete variable, merge the referencing variable with the
    referenced variable.

    When using the shared variable table, the following variables:

    ::

        T x = { 3, 7 };
        T* p = &x;
        T& r = x;

        { name: "x", var_table_index: 3, type: "T" }  // Captured variables
        { name: "p", value "0x00500500", type="T*", var_table_index: 3 }
        { name: "r", type="T&", var_table_index: 3 }

        {  // Shared variable table entry #3:
            members { name: "m1", value: "3", type: "int" },
            members { name: "m2", value: "7", type: "int" }
        }

    Note that the pointer address is stored with the referencing
    variable and not with the referenced variable. This allows the
    referenced variable to be shared between pointers and references.

    The type field is optional. The debugger agent may or may not
    support it.

    Attributes:
        name (str):
            Name of the variable, if any.
        value (str):
            Simple value of the variable.
        type_ (str):
            Variable type (e.g. ``MyClass``). If the variable is split
            with ``var_table_index``, ``type`` goes next to ``value``.
            The interpretation of a type is agent specific. It is
            recommended to include the dynamic type rather than a static
            type of an object.
        members (Sequence[google.cloud.debugger_v2.types.Variable]):
            Members contained or pointed to by the
            variable.
        var_table_index (google.protobuf.wrappers_pb2.Int32Value):
            Reference to a variable in the shared variable table. More
            than one variable can reference the same variable in the
            table. The ``var_table_index`` field is an index into
            ``variable_table`` in Breakpoint.
        status (google.cloud.debugger_v2.types.StatusMessage):
            Status associated with the variable. This field will usually
            stay unset. A status of a single variable only applies to
            that variable or expression. The rest of breakpoint data
            still remains valid. Variables might be reported in error
            state even when breakpoint is not in final state.

            The message may refer to variable name with ``refers_to``
            set to ``VARIABLE_NAME``. Alternatively ``refers_to`` will
            be set to ``VARIABLE_VALUE``. In either case variable value
            and members will be unset.

            Example of error message applied to name:
            ``Invalid expression syntax``.

            Example of information message applied to value:
            ``Not captured``.

            Examples of error message applied to value:

            -  ``Malformed string``,
            -  ``Field f not found in class C``
            -  ``Null pointer dereference``
    """

    name = proto.Field(
        proto.STRING,
        number=1,
    )
    value = proto.Field(
        proto.STRING,
        number=2,
    )
    type_ = proto.Field(
        proto.STRING,
        number=6,
    )
    members = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="Variable",
    )
    var_table_index = proto.Field(
        proto.MESSAGE,
        number=4,
        message=wrappers_pb2.Int32Value,
    )
    status = proto.Field(
        proto.MESSAGE,
        number=5,
        message="StatusMessage",
    )


class StackFrame(proto.Message):
    r"""Represents a stack frame context.

    Attributes:
        function (str):
            Demangled function name at the call site.
        location (google.cloud.debugger_v2.types.SourceLocation):
            Source location of the call site.
        arguments (Sequence[google.cloud.debugger_v2.types.Variable]):
            Set of arguments passed to this function.
            Note that this might not be populated for all
            stack frames.
        locals_ (Sequence[google.cloud.debugger_v2.types.Variable]):
            Set of local variables at the stack frame
            location. Note that this might not be populated
            for all stack frames.
    """

    function = proto.Field(
        proto.STRING,
        number=1,
    )
    location = proto.Field(
        proto.MESSAGE,
        number=2,
        message="SourceLocation",
    )
    arguments = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="Variable",
    )
    locals_ = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="Variable",
    )


class Breakpoint(proto.Message):
    r"""Represents the breakpoint specification, status and results.

    Attributes:
        id (str):
            Breakpoint identifier, unique in the scope of
            the debuggee.
        action (google.cloud.debugger_v2.types.Breakpoint.Action):
            Action that the agent should perform when the
            code at the breakpoint location is hit.
        location (google.cloud.debugger_v2.types.SourceLocation):
            Breakpoint source location.
        condition (str):
            Condition that triggers the breakpoint.
            The condition is a compound boolean expression
            composed using expressions in a programming
            language at the source location.
        expressions (Sequence[str]):
            List of read-only expressions to evaluate at the breakpoint
            location. The expressions are composed using expressions in
            the programming language at the source location. If the
            breakpoint action is ``LOG``, the evaluated expressions are
            included in log statements.
        log_message_format (str):
            Only relevant when action is ``LOG``. Defines the message to
            log when the breakpoint hits. The message may include
            parameter placeholders ``$0``, ``$1``, etc. These
            placeholders are replaced with the evaluated value of the
            appropriate expression. Expressions not referenced in
            ``log_message_format`` are not logged.

            Example: ``Message received, id = $0, count = $1`` with
            ``expressions`` = ``[ message.id, message.count ]``.
        log_level (google.cloud.debugger_v2.types.Breakpoint.LogLevel):
            Indicates the severity of the log. Only relevant when action
            is ``LOG``.
        is_final_state (bool):
            When true, indicates that this is a final
            result and the breakpoint state will not change
            from here on.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Time this breakpoint was created by the
            server in seconds resolution.
        final_time (google.protobuf.timestamp_pb2.Timestamp):
            Time this breakpoint was finalized as seen by
            the server in seconds resolution.
        user_email (str):
            E-mail address of the user that created this
            breakpoint
        status (google.cloud.debugger_v2.types.StatusMessage):
            Breakpoint status.

            The status includes an error flag and a human readable
            message. This field is usually unset. The message can be
            either informational or an error message. Regardless,
            clients should always display the text message back to the
            user.

            Error status indicates complete failure of the breakpoint.

            Example (non-final state): ``Still loading symbols...``

            Examples (final state):

            -  ``Invalid line number`` referring to location
            -  ``Field f not found in class C`` referring to condition
        stack_frames (Sequence[google.cloud.debugger_v2.types.StackFrame]):
            The stack at breakpoint time, where stack_frames[0]
            represents the most recently entered function.
        evaluated_expressions (Sequence[google.cloud.debugger_v2.types.Variable]):
            Values of evaluated expressions at breakpoint time. The
            evaluated expressions appear in exactly the same order they
            are listed in the ``expressions`` field. The ``name`` field
            holds the original expression text, the ``value`` or
            ``members`` field holds the result of the evaluated
            expression. If the expression cannot be evaluated, the
            ``status`` inside the ``Variable`` will indicate an error
            and contain the error text.
        variable_table (Sequence[google.cloud.debugger_v2.types.Variable]):
            The ``variable_table`` exists to aid with computation,
            memory and network traffic optimization. It enables storing
            a variable once and reference it from multiple variables,
            including variables stored in the ``variable_table`` itself.
            For example, the same ``this`` object, which may appear at
            many levels of the stack, can have all of its data stored
            once in this table. The stack frame variables then would
            hold only a reference to it.

            The variable ``var_table_index`` field is an index into this
            repeated field. The stored objects are nameless and get
            their name from the referencing variable. The effective
            variable is a merge of the referencing variable and the
            referenced variable.
        labels (Mapping[str, str]):
            A set of custom breakpoint properties,
            populated by the agent, to be displayed to the
            user.
    """

    class Action(proto.Enum):
        r"""Actions that can be taken when a breakpoint hits.
        Agents should reject breakpoints with unsupported or unknown
        action values.
        """
        CAPTURE = 0
        LOG = 1

    class LogLevel(proto.Enum):
        r"""Log severity levels."""
        INFO = 0
        WARNING = 1
        ERROR = 2

    id = proto.Field(
        proto.STRING,
        number=1,
    )
    action = proto.Field(
        proto.ENUM,
        number=13,
        enum=Action,
    )
    location = proto.Field(
        proto.MESSAGE,
        number=2,
        message="SourceLocation",
    )
    condition = proto.Field(
        proto.STRING,
        number=3,
    )
    expressions = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    log_message_format = proto.Field(
        proto.STRING,
        number=14,
    )
    log_level = proto.Field(
        proto.ENUM,
        number=15,
        enum=LogLevel,
    )
    is_final_state = proto.Field(
        proto.BOOL,
        number=5,
    )
    create_time = proto.Field(
        proto.MESSAGE,
        number=11,
        message=timestamp_pb2.Timestamp,
    )
    final_time = proto.Field(
        proto.MESSAGE,
        number=12,
        message=timestamp_pb2.Timestamp,
    )
    user_email = proto.Field(
        proto.STRING,
        number=16,
    )
    status = proto.Field(
        proto.MESSAGE,
        number=10,
        message="StatusMessage",
    )
    stack_frames = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="StackFrame",
    )
    evaluated_expressions = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="Variable",
    )
    variable_table = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message="Variable",
    )
    labels = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=17,
    )


class Debuggee(proto.Message):
    r"""Represents the debugged application. The application may
    include one or more replicated processes executing the same
    code. Each of these processes is attached with a debugger agent,
    carrying out the debugging commands. Agents attached to the same
    debuggee identify themselves as such by using exactly the same
    Debuggee message value when registering.

    Attributes:
        id (str):
            Unique identifier for the debuggee generated
            by the controller service.
        project (str):
            Project the debuggee is associated with.
            Use project number or id when registering a
            Google Cloud Platform project.
        uniquifier (str):
            Uniquifier to further distinguish the
            application. It is possible that different
            applications might have identical values in the
            debuggee message, thus, incorrectly identified
            as a single application by the Controller
            service. This field adds salt to further
            distinguish the application. Agents should
            consider seeding this field with value that
            identifies the code, binary, configuration and
            environment.
        description (str):
            Human readable description of the debuggee.
            Including a human-readable project name,
            environment name and version information is
            recommended.
        is_inactive (bool):
            If set to ``true``, indicates that Controller service does
            not detect any activity from the debuggee agents and the
            application is possibly stopped.
        agent_version (str):
            Version ID of the agent. Schema:
            ``domain/language-platform/vmajor.minor`` (for example
            ``google.com/java-gcp/v1.1``).
        is_disabled (bool):
            If set to ``true``, indicates that the agent should disable
            itself and detach from the debuggee.
        status (google.cloud.debugger_v2.types.StatusMessage):
            Human readable message to be displayed to the
            user about this debuggee. Absence of this field
            indicates no status. The message can be either
            informational or an error status.
        source_contexts (Sequence[google.cloud.source_context_v1.types.source_context_pb2.SourceContext]):
            References to the locations and revisions of
            the source code used in the deployed
            application.
        ext_source_contexts (Sequence[google.cloud.source_context_v1.types.source_context_pb2.ExtendedSourceContext]):
            References to the locations and revisions of
            the source code used in the deployed
            application.
        labels (Mapping[str, str]):
            A set of custom debuggee properties,
            populated by the agent, to be displayed to the
            user.
    """

    id = proto.Field(
        proto.STRING,
        number=1,
    )
    project = proto.Field(
        proto.STRING,
        number=2,
    )
    uniquifier = proto.Field(
        proto.STRING,
        number=3,
    )
    description = proto.Field(
        proto.STRING,
        number=4,
    )
    is_inactive = proto.Field(
        proto.BOOL,
        number=5,
    )
    agent_version = proto.Field(
        proto.STRING,
        number=6,
    )
    is_disabled = proto.Field(
        proto.BOOL,
        number=7,
    )
    status = proto.Field(
        proto.MESSAGE,
        number=8,
        message="StatusMessage",
    )
    source_contexts = proto.RepeatedField(
        proto.MESSAGE,
        number=9,
        message=source_context_pb2.SourceContext,
    )
    ext_source_contexts = proto.RepeatedField(
        proto.MESSAGE,
        number=13,
        message=source_context_pb2.ExtendedSourceContext,
    )
    labels = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=11,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
import pkg_resources

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.debugger_v2.types import controller
from google.cloud.debugger_v2.types import data
from .transports.base import Controller2Transport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import Controller2GrpcAsyncIOTransport
from .client import Controller2Client


class Controller2AsyncClient:
    """The Controller service provides the API for orchestrating a
    collection of debugger agents to perform debugging tasks. These
    agents are each attached to a process of an application which may
    include one or more replicas.

    The debugger agents register with the Controller to identify the
    application being debugged, the Debuggee. All agents that register
    with the same data, represent the same Debuggee, and are assigned
    the same ``debuggee_id``.

    The debugger agents call the Controller to retrieve the list of
    active Breakpoints. Agents with the same ``debuggee_id`` get the
    same breakpoints list. An agent that can fulfill the breakpoint
    request updates the Controller with the breakpoint result. The
    controller selects the first result received and discards the rest
    of the results. Agents that poll again for active breakpoints will
    no longer have the completed breakpoint in the list and should
    remove that breakpoint from their attached process.

    The Controller service does not provide a way to retrieve the
    results of a completed breakpoint. This functionality is available
    using the Debugger service.
    """

    _client: Controller2Client

    DEFAULT_ENDPOINT = Controller2Client.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = Controller2Client.DEFAULT_MTLS_ENDPOINT

    common_billing_account_path = staticmethod(
        Controller2Client.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        Controller2Client.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(Controller2Client.common_folder_path)
    parse_common_folder_path = staticmethod(Controller2Client.parse_common_folder_path)
    common_organization_path = staticmethod(Controller2Client.common_organization_path)
    parse_common_organization_path = staticmethod(
        Controller2Client.parse_common_organization_path
    )
    common_project_path = staticmethod(Controller2Client.common_project_path)
    parse_common_project_path = staticmethod(
        Controller2Client.parse_common_project_path
    )
    common_location_path = staticmethod(Controller2Client.common_location_path)
    parse_common_location_path = staticmethod(
        Controller2Client.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            Controller2AsyncClient: The constructed client.
        """
        return Controller2Client.from_service_account_info.__func__(Controller2AsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            Controller2AsyncClient: The constructed client.
        """
        return Controller2Client.from_service_account_file.__func__(Controller2AsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variabel is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return Controller2Client.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> Controller2Transport:
        """Returns the transport used by the client instance.

        Returns:
            Controller2Transport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(Controller2Client).get_transport_class, type(Controller2Client)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, Controller2Transport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the controller2 client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.Controller2Transport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = Controller2Client(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def register_debuggee(
        self,
        request: Union[controller.RegisterDebuggeeRequest, dict] = None,
        *,
        debuggee: data.Debuggee = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> controller.RegisterDebuggeeResponse:
        r"""Registers the debuggee with the controller service.

        All agents attached to the same application must call this
        method with exactly the same request content to get back the
        same stable ``debuggee_id``. Agents should call this method
        again whenever ``google.rpc.Code.NOT_FOUND`` is returned from
        any controller method.

        This protocol allows the controller service to disable
        debuggees, recover from data loss, or change the ``debuggee_id``
        format. Agents must handle ``debuggee_id`` value changing upon
        re-registration.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import debugger_v2

            async def sample_register_debuggee():
                # Create a client
                client = debugger_v2.Controller2AsyncClient()

                # Initialize request argument(s)
                request = debugger_v2.RegisterDebuggeeRequest(
                )

                # Make the request
                response = await client.register_debuggee(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.debugger_v2.types.RegisterDebuggeeRequest, dict]):
                The request object. Request to register a debuggee.
            debuggee (:class:`google.cloud.debugger_v2.types.Debuggee`):
                Required. Debuggee information to register. The fields
                ``project``, ``uniquifier``, ``description`` and
                ``agent_version`` of the debuggee must be set.

                This corresponds to the ``debuggee`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.debugger_v2.types.RegisterDebuggeeResponse:
                Response for registering a debuggee.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([debuggee])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = controller.RegisterDebuggeeRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if debuggee is not None:
            request.debuggee = debuggee

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.register_debuggee,
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def list_active_breakpoints(
        self,
        request: Union[controller.ListActiveBreakpointsRequest, dict] = None,
        *,
        debuggee_id: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> controller.ListActiveBreakpointsResponse:
        r"""Returns the list of all active breakpoints for the debuggee.

        The breakpoint specification (``location``, ``condition``, and
        ``expressions`` fields) is semantically immutable, although the
        field values may change. For example, an agent may update the
        location line number to reflect the actual line where the
        breakpoint was set, but this doesn't change the breakpoint
        semantics.

        This means that an agent does not need to check if a breakpoint
        has changed when it encounters the same breakpoint on a
        successive call. Moreover, an agent should remember the
        breakpoints that are completed until the controller removes them
        from the active list to avoid setting those breakpoints again.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import debugger_v2

            async def sample_list_active_breakpoints():
                # Create a client
                client = debugger_v2.Controller2AsyncClient()

                # Initialize request argument(s)
                request = debugger_v2.ListActiveBreakpointsRequest(
                    debuggee_id="debuggee_id_value",
                )

                # Make the request
                response = await client.list_active_breakpoints(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.debugger_v2.types.ListActiveBreakpointsRequest, dict]):
                The request object. Request to list active breakpoints.
            debuggee_id (:class:`str`):
                Required. Identifies the debuggee.
                This corresponds to the ``debuggee_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.debugger_v2.types.ListActiveBreakpointsResponse:
                Response for listing active
                breakpoints.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([debuggee_id])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = controller.ListActiveBreakpointsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if debuggee_id is not None:
            request.debuggee_id = debuggee_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_active_breakpoints,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=600.0,
            ),
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("debuggee_id", request.debuggee_id),)
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def update_active_breakpoint(
        self,
        request: Union[controller.UpdateActiveBreakpointRequest, dict] = None,
        *,
        debuggee_id: str = None,
        breakpoint_: data.Breakpoint = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> controller.UpdateActiveBreakpointResponse:
        r"""Updates the breakpoint state or mutable fields. The entire
        Breakpoint message must be sent back to the controller service.

        Updates to active breakpoint fields are only allowed if the new
        value does not change the breakpoint specification. Updates to
        the ``location``, ``condition`` and ``expressions`` fields
        should not alter the breakpoint semantics. These may only make
        changes such as canonicalizing a value or snapping the location
        to the correct line of code.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.cloud import debugger_v2

            async def sample_update_active_breakpoint():
                # Create a client
                client = debugger_v2.Controller2AsyncClient()

                # Initialize request argument(s)
                request = debugger_v2.UpdateActiveBreakpointRequest(
                    debuggee_id="debuggee_id_value",
                )

                # Make the request
                response = await client.update_active_breakpoint(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.debugger_v2.types.UpdateActiveBreakpointRequest, dict]):
                The request object. Request to update an active
                breakpoint.
            debuggee_id (:class:`str`):
                Required. Identifies the debuggee
                being debugged.

                This corresponds to the ``debuggee_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            breakpoint_ (:class:`google.cloud.debugger_v2.types.Breakpoint`):
                Required. Updated breakpoint information. The field
                ``id`` must be set. The agent must echo all Breakpoint
                specification fields in the update.

                This corresponds to the ``breakpoint_`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.debugger_v2.types.UpdateActiveBreakpointResponse:
                Response for updating an active
                breakpoint. The message is defined to
                allow future extensions.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([debuggee_id, breakpoint_])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = controller.UpdateActiveBreakpointRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if debuggee_id is not None:
            request.debuggee_id = debuggee_id
        if breakpoint_ is not None:
            request.breakpoint_ = breakpoint_

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_active_breakpoint,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.DeadlineExceeded,
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=600.0,
            ),
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (
                    ("debuggee_id", request.debuggee_id),
                    ("breakpoint.id", request.breakpoint_.id),
                )
            ),
        )

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-debugger-client",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("Controller2AsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .controller import (
    ListActiveBreakpointsRequest,
    ListActiveBreakpointsResponse,
    RegisterDebuggeeRequest,
    RegisterDebuggeeResponse,
    UpdateActiveBreakpointRequest,
    UpdateActiveBreakpointResponse,
)
from .data import (
    Breakpoint,
    Debuggee,
    FormatMessage,
    SourceLocation,
    StackFrame,
    StatusMessage,
    Variable,
)
from .debugger import (
    DeleteBreakpointRequest,
    GetBreakpointRequest,
    GetBreakpointResponse,
    ListBreakpointsRequest,
    ListBreakpointsResponse,
    ListDebuggeesRequest,
    ListDebuggeesResponse,
    SetBreakpointRequest,
    SetBreakpointResponse,
)

__all__ = (
    "ListActiveBreakpointsRequest",
    "ListActiveBreakpointsResponse",
    "RegisterDebuggeeRequest",
    "RegisterDebuggeeResponse",
    "UpdateActiveBreakpointRequest",
    "UpdateActiveBreakpointResponse",
    "Breakpoint",
    "Debuggee",
    "FormatMessage",
    "SourceLocation",
    "StackFrame",
    "StatusMessage",
    "Variable",
    "DeleteBreakpointRequest",
    "GetBreakpointRequest",
    "GetBreakpointResponse",
    "ListBreakpointsRequest",
    "ListBreakpointsResponse",
    "ListDebuggeesRequest",
    "ListDebuggeesResponse",
    "SetBreakpointRequest",
    "SetBreakpointResponse",
)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.debugger_v2.types import data


__protobuf__ = proto.module(
    package="google.devtools.clouddebugger.v2",
    manifest={
        "SetBreakpointRequest",
        "SetBreakpointResponse",
        "GetBreakpointRequest",
        "GetBreakpointResponse",
        "DeleteBreakpointRequest",
        "ListBreakpointsRequest",
        "ListBreakpointsResponse",
        "ListDebuggeesRequest",
        "ListDebuggeesResponse",
    },
)


class SetBreakpointRequest(proto.Message):
    r"""Request to set a breakpoint

    Attributes:
        debuggee_id (str):
            Required. ID of the debuggee where the
            breakpoint is to be set.
        breakpoint_ (google.cloud.debugger_v2.types.Breakpoint):
            Required. Breakpoint specification to set. The field
            ``location`` of the breakpoint must be set.
        client_version (str):
            Required. The client version making the call. Schema:
            ``domain/type/version`` (e.g., ``google.com/intellij/v1``).
    """

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    breakpoint_ = proto.Field(
        proto.MESSAGE,
        number=2,
        message=data.Breakpoint,
    )
    client_version = proto.Field(
        proto.STRING,
        number=4,
    )


class SetBreakpointResponse(proto.Message):
    r"""Response for setting a breakpoint.

    Attributes:
        breakpoint_ (google.cloud.debugger_v2.types.Breakpoint):
            Breakpoint resource. The field ``id`` is guaranteed to be
            set (in addition to the echoed fileds).
    """

    breakpoint_ = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.Breakpoint,
    )


class GetBreakpointRequest(proto.Message):
    r"""Request to get breakpoint information.

    Attributes:
        debuggee_id (str):
            Required. ID of the debuggee whose breakpoint
            to get.
        breakpoint_id (str):
            Required. ID of the breakpoint to get.
        client_version (str):
            Required. The client version making the call. Schema:
            ``domain/type/version`` (e.g., ``google.com/intellij/v1``).
    """

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    breakpoint_id = proto.Field(
        proto.STRING,
        number=2,
    )
    client_version = proto.Field(
        proto.STRING,
        number=4,
    )


class GetBreakpointResponse(proto.Message):
    r"""Response for getting breakpoint information.

    Attributes:
        breakpoint_ (google.cloud.debugger_v2.types.Breakpoint):
            Complete breakpoint state. The fields ``id`` and
            ``location`` are guaranteed to be set.
    """

    breakpoint_ = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.Breakpoint,
    )


class DeleteBreakpointRequest(proto.Message):
    r"""Request to delete a breakpoint.

    Attributes:
        debuggee_id (str):
            Required. ID of the debuggee whose breakpoint
            to delete.
        breakpoint_id (str):
            Required. ID of the breakpoint to delete.
        client_version (str):
            Required. The client version making the call. Schema:
            ``domain/type/version`` (e.g., ``google.com/intellij/v1``).
    """

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    breakpoint_id = proto.Field(
        proto.STRING,
        number=2,
    )
    client_version = proto.Field(
        proto.STRING,
        number=3,
    )


class ListBreakpointsRequest(proto.Message):
    r"""Request to list breakpoints.

    Attributes:
        debuggee_id (str):
            Required. ID of the debuggee whose
            breakpoints to list.
        include_all_users (bool):
            When set to ``true``, the response includes the list of
            breakpoints set by any user. Otherwise, it includes only
            breakpoints set by the caller.
        include_inactive (bool):
            When set to ``true``, the response includes active and
            inactive breakpoints. Otherwise, it includes only active
            breakpoints.
        action (google.cloud.debugger_v2.types.ListBreakpointsRequest.BreakpointActionValue):
            When set, the response includes only
            breakpoints with the specified action.
        strip_results (bool):
            This field is deprecated. The following fields are always
            stripped out of the result: ``stack_frames``,
            ``evaluated_expressions`` and ``variable_table``.
        wait_token (str):
            A wait token that, if specified, blocks the call until the
            breakpoints list has changed, or a server selected timeout
            has expired. The value should be set from the last response.
            The error code ``google.rpc.Code.ABORTED`` (RPC) is returned
            on wait timeout, which should be called again with the same
            ``wait_token``.
        client_version (str):
            Required. The client version making the call. Schema:
            ``domain/type/version`` (e.g., ``google.com/intellij/v1``).
    """

    class BreakpointActionValue(proto.Message):
        r"""Wrapper message for ``Breakpoint.Action``. Defines a filter on the
        action field of breakpoints.

        Attributes:
            value (google.cloud.debugger_v2.types.Breakpoint.Action):
                Only breakpoints with the specified action
                will pass the filter.
        """

        value = proto.Field(
            proto.ENUM,
            number=1,
            enum=data.Breakpoint.Action,
        )

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    include_all_users = proto.Field(
        proto.BOOL,
        number=2,
    )
    include_inactive = proto.Field(
        proto.BOOL,
        number=3,
    )
    action = proto.Field(
        proto.MESSAGE,
        number=4,
        message=BreakpointActionValue,
    )
    strip_results = proto.Field(
        proto.BOOL,
        number=5,
    )
    wait_token = proto.Field(
        proto.STRING,
        number=6,
    )
    client_version = proto.Field(
        proto.STRING,
        number=8,
    )


class ListBreakpointsResponse(proto.Message):
    r"""Response for listing breakpoints.

    Attributes:
        breakpoints (Sequence[google.cloud.debugger_v2.types.Breakpoint]):
            List of breakpoints matching the request. The fields ``id``
            and ``location`` are guaranteed to be set on each
            breakpoint. The fields: ``stack_frames``,
            ``evaluated_expressions`` and ``variable_table`` are cleared
            on each breakpoint regardless of its status.
        next_wait_token (str):
            A wait token that can be used in the next call to ``list``
            (REST) or ``ListBreakpoints`` (RPC) to block until the list
            of breakpoints has changes.
    """

    breakpoints = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=data.Breakpoint,
    )
    next_wait_token = proto.Field(
        proto.STRING,
        number=2,
    )


class ListDebuggeesRequest(proto.Message):
    r"""Request to list debuggees.

    Attributes:
        project (str):
            Required. Project number of a Google Cloud
            project whose debuggees to list.
        include_inactive (bool):
            When set to ``true``, the result includes all debuggees.
            Otherwise, the result includes only debuggees that are
            active.
        client_version (str):
            Required. The client version making the call. Schema:
            ``domain/type/version`` (e.g., ``google.com/intellij/v1``).
    """

    project = proto.Field(
        proto.STRING,
        number=2,
    )
    include_inactive = proto.Field(
        proto.BOOL,
        number=3,
    )
    client_version = proto.Field(
        proto.STRING,
        number=4,
    )


class ListDebuggeesResponse(proto.Message):
    r"""Response for listing debuggees.

    Attributes:
        debuggees (Sequence[google.cloud.debugger_v2.types.Debuggee]):
            List of debuggees accessible to the calling user. The fields
            ``debuggee.id`` and ``description`` are guaranteed to be
            set. The ``description`` field is a human readable field
            provided by agents and can be displayed to users.
    """

    debuggees = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=data.Debuggee,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union

from google.api_core import gapic_v1
from google.api_core import grpc_helpers_async
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore

import grpc  # type: ignore
from grpc.experimental import aio  # type: ignore

from google.cloud.debugger_v2.types import controller
from .base import Controller2Transport, DEFAULT_CLIENT_INFO
from .grpc import Controller2GrpcTransport


class Controller2GrpcAsyncIOTransport(Controller2Transport):
    """gRPC AsyncIO backend transport for Controller2.

    The Controller service provides the API for orchestrating a
    collection of debugger agents to perform debugging tasks. These
    agents are each attached to a process of an application which may
    include one or more replicas.

    The debugger agents register with the Controller to identify the
    application being debugged, the Debuggee. All agents that register
    with the same data, represent the same Debuggee, and are assigned
    the same ``debuggee_id``.

    The debugger agents call the Controller to retrieve the list of
    active Breakpoints. Agents with the same ``debuggee_id`` get the
    same breakpoints list. An agent that can fulfill the breakpoint
    request updates the Controller with the breakpoint result. The
    controller selects the first result received and discards the rest
    of the results. Agents that poll again for active breakpoints will
    no longer have the completed breakpoint in the list and should
    remove that breakpoint from their attached process.

    The Controller service does not provide a way to retrieve the
    results of a completed breakpoint. This functionality is available
    using the Debugger service.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _grpc_channel: aio.Channel
    _stubs: Dict[str, Callable] = {}

    @classmethod
    def create_channel(
        cls,
        host: str = "clouddebugger.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> aio.Channel:
        """Create and return a gRPC AsyncIO channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            aio.Channel: A gRPC AsyncIO channel object.
        """

        return grpc_helpers_async.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    def __init__(
        self,
        *,
        host: str = "clouddebugger.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        channel: aio.Channel = None,
        api_mtls_endpoint: str = None,
        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
        ssl_channel_credentials: grpc.ChannelCredentials = None,
        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
        quota_project_id=None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            channel (Optional[aio.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None
        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @property
    def grpc_channel(self) -> aio.Channel:
        """Create the channel designed to connect to this service.

        This property caches on the instance; repeated calls return
        the same channel.
        """
        # Return the channel from cache.
        return self._grpc_channel

    @property
    def register_debuggee(
        self,
    ) -> Callable[
        [controller.RegisterDebuggeeRequest],
        Awaitable[controller.RegisterDebuggeeResponse],
    ]:
        r"""Return a callable for the register debuggee method over gRPC.

        Registers the debuggee with the controller service.

        All agents attached to the same application must call this
        method with exactly the same request content to get back the
        same stable ``debuggee_id``. Agents should call this method
        again whenever ``google.rpc.Code.NOT_FOUND`` is returned from
        any controller method.

        This protocol allows the controller service to disable
        debuggees, recover from data loss, or change the ``debuggee_id``
        format. Agents must handle ``debuggee_id`` value changing upon
        re-registration.

        Returns:
            Callable[[~.RegisterDebuggeeRequest],
                    Awaitable[~.RegisterDebuggeeResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "register_debuggee" not in self._stubs:
            self._stubs["register_debuggee"] = self.grpc_channel.unary_unary(
                "/google.devtools.clouddebugger.v2.Controller2/RegisterDebuggee",
                request_serializer=controller.RegisterDebuggeeRequest.serialize,
                response_deserializer=controller.RegisterDebuggeeResponse.deserialize,
            )
        return self._stubs["register_debuggee"]

    @property
    def list_active_breakpoints(
        self,
    ) -> Callable[
        [controller.ListActiveBreakpointsRequest],
        Awaitable[controller.ListActiveBreakpointsResponse],
    ]:
        r"""Return a callable for the list active breakpoints method over gRPC.

        Returns the list of all active breakpoints for the debuggee.

        The breakpoint specification (``location``, ``condition``, and
        ``expressions`` fields) is semantically immutable, although the
        field values may change. For example, an agent may update the
        location line number to reflect the actual line where the
        breakpoint was set, but this doesn't change the breakpoint
        semantics.

        This means that an agent does not need to check if a breakpoint
        has changed when it encounters the same breakpoint on a
        successive call. Moreover, an agent should remember the
        breakpoints that are completed until the controller removes them
        from the active list to avoid setting those breakpoints again.

        Returns:
            Callable[[~.ListActiveBreakpointsRequest],
                    Awaitable[~.ListActiveBreakpointsResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_active_breakpoints" not in self._stubs:
            self._stubs["list_active_breakpoints"] = self.grpc_channel.unary_unary(
                "/google.devtools.clouddebugger.v2.Controller2/ListActiveBreakpoints",
                request_serializer=controller.ListActiveBreakpointsRequest.serialize,
                response_deserializer=controller.ListActiveBreakpointsResponse.deserialize,
            )
        return self._stubs["list_active_breakpoints"]

    @property
    def update_active_breakpoint(
        self,
    ) -> Callable[
        [controller.UpdateActiveBreakpointRequest],
        Awaitable[controller.UpdateActiveBreakpointResponse],
    ]:
        r"""Return a callable for the update active breakpoint method over gRPC.

        Updates the breakpoint state or mutable fields. The entire
        Breakpoint message must be sent back to the controller service.

        Updates to active breakpoint fields are only allowed if the new
        value does not change the breakpoint specification. Updates to
        the ``location``, ``condition`` and ``expressions`` fields
        should not alter the breakpoint semantics. These may only make
        changes such as canonicalizing a value or snapping the location
        to the correct line of code.

        Returns:
            Callable[[~.UpdateActiveBreakpointRequest],
                    Awaitable[~.UpdateActiveBreakpointResponse]]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_active_breakpoint" not in self._stubs:
            self._stubs["update_active_breakpoint"] = self.grpc_channel.unary_unary(
                "/google.devtools.clouddebugger.v2.Controller2/UpdateActiveBreakpoint",
                request_serializer=controller.UpdateActiveBreakpointRequest.serialize,
                response_deserializer=controller.UpdateActiveBreakpointResponse.deserialize,
            )
        return self._stubs["update_active_breakpoint"]

    def close(self):
        return self.grpc_channel.close()


__all__ = ("Controller2GrpcAsyncIOTransport",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.debugger_v2.services.controller2.client import Controller2Client
from google.cloud.debugger_v2.services.controller2.async_client import (
    Controller2AsyncClient,
)
from google.cloud.debugger_v2.services.debugger2.client import Debugger2Client
from google.cloud.debugger_v2.services.debugger2.async_client import (
    Debugger2AsyncClient,
)

from google.cloud.debugger_v2.types.controller import ListActiveBreakpointsRequest
from google.cloud.debugger_v2.types.controller import ListActiveBreakpointsResponse
from google.cloud.debugger_v2.types.controller import RegisterDebuggeeRequest
from google.cloud.debugger_v2.types.controller import RegisterDebuggeeResponse
from google.cloud.debugger_v2.types.controller import UpdateActiveBreakpointRequest
from google.cloud.debugger_v2.types.controller import UpdateActiveBreakpointResponse
from google.cloud.debugger_v2.types.data import Breakpoint
from google.cloud.debugger_v2.types.data import Debuggee
from google.cloud.debugger_v2.types.data import FormatMessage
from google.cloud.debugger_v2.types.data import SourceLocation
from google.cloud.debugger_v2.types.data import StackFrame
from google.cloud.debugger_v2.types.data import StatusMessage
from google.cloud.debugger_v2.types.data import Variable
from google.cloud.debugger_v2.types.debugger import DeleteBreakpointRequest
from google.cloud.debugger_v2.types.debugger import GetBreakpointRequest
from google.cloud.debugger_v2.types.debugger import GetBreakpointResponse
from google.cloud.debugger_v2.types.debugger import ListBreakpointsRequest
from google.cloud.debugger_v2.types.debugger import ListBreakpointsResponse
from google.cloud.debugger_v2.types.debugger import ListDebuggeesRequest
from google.cloud.debugger_v2.types.debugger import ListDebuggeesResponse
from google.cloud.debugger_v2.types.debugger import SetBreakpointRequest
from google.cloud.debugger_v2.types.debugger import SetBreakpointResponse

__all__ = (
    "Controller2Client",
    "Controller2AsyncClient",
    "Debugger2Client",
    "Debugger2AsyncClient",
    "ListActiveBreakpointsRequest",
    "ListActiveBreakpointsResponse",
    "RegisterDebuggeeRequest",
    "RegisterDebuggeeResponse",
    "UpdateActiveBreakpointRequest",
    "UpdateActiveBreakpointResponse",
    "Breakpoint",
    "Debuggee",
    "FormatMessage",
    "SourceLocation",
    "StackFrame",
    "StatusMessage",
    "Variable",
    "DeleteBreakpointRequest",
    "GetBreakpointRequest",
    "GetBreakpointResponse",
    "ListBreakpointsRequest",
    "ListBreakpointsResponse",
    "ListDebuggeesRequest",
    "ListDebuggeesResponse",
    "SetBreakpointRequest",
    "SetBreakpointResponse",
)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.controller2 import Controller2Client
from .services.controller2 import Controller2AsyncClient
from .services.debugger2 import Debugger2Client
from .services.debugger2 import Debugger2AsyncClient

from .types.controller import ListActiveBreakpointsRequest
from .types.controller import ListActiveBreakpointsResponse
from .types.controller import RegisterDebuggeeRequest
from .types.controller import RegisterDebuggeeResponse
from .types.controller import UpdateActiveBreakpointRequest
from .types.controller import UpdateActiveBreakpointResponse
from .types.data import Breakpoint
from .types.data import Debuggee
from .types.data import FormatMessage
from .types.data import SourceLocation
from .types.data import StackFrame
from .types.data import StatusMessage
from .types.data import Variable
from .types.debugger import DeleteBreakpointRequest
from .types.debugger import GetBreakpointRequest
from .types.debugger import GetBreakpointResponse
from .types.debugger import ListBreakpointsRequest
from .types.debugger import ListBreakpointsResponse
from .types.debugger import ListDebuggeesRequest
from .types.debugger import ListDebuggeesResponse
from .types.debugger import SetBreakpointRequest
from .types.debugger import SetBreakpointResponse

__all__ = (
    "Controller2AsyncClient",
    "Debugger2AsyncClient",
    "Breakpoint",
    "Controller2Client",
    "Debuggee",
    "Debugger2Client",
    "DeleteBreakpointRequest",
    "FormatMessage",
    "GetBreakpointRequest",
    "GetBreakpointResponse",
    "ListActiveBreakpointsRequest",
    "ListActiveBreakpointsResponse",
    "ListBreakpointsRequest",
    "ListBreakpointsResponse",
    "ListDebuggeesRequest",
    "ListDebuggeesResponse",
    "RegisterDebuggeeRequest",
    "RegisterDebuggeeResponse",
    "SetBreakpointRequest",
    "SetBreakpointResponse",
    "SourceLocation",
    "StackFrame",
    "StatusMessage",
    "UpdateActiveBreakpointRequest",
    "UpdateActiveBreakpointResponse",
    "Variable",
)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.cloud.debugger_v2.types import data


__protobuf__ = proto.module(
    package="google.devtools.clouddebugger.v2",
    manifest={
        "RegisterDebuggeeRequest",
        "RegisterDebuggeeResponse",
        "ListActiveBreakpointsRequest",
        "ListActiveBreakpointsResponse",
        "UpdateActiveBreakpointRequest",
        "UpdateActiveBreakpointResponse",
    },
)


class RegisterDebuggeeRequest(proto.Message):
    r"""Request to register a debuggee.

    Attributes:
        debuggee (google.cloud.debugger_v2.types.Debuggee):
            Required. Debuggee information to register. The fields
            ``project``, ``uniquifier``, ``description`` and
            ``agent_version`` of the debuggee must be set.
    """

    debuggee = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.Debuggee,
    )


class RegisterDebuggeeResponse(proto.Message):
    r"""Response for registering a debuggee.

    Attributes:
        debuggee (google.cloud.debugger_v2.types.Debuggee):
            Debuggee resource. The field ``id`` is guaranteed to be set
            (in addition to the echoed fields). If the field
            ``is_disabled`` is set to ``true``, the agent should disable
            itself by removing all breakpoints and detaching from the
            application. It should however continue to poll
            ``RegisterDebuggee`` until reenabled.
    """

    debuggee = proto.Field(
        proto.MESSAGE,
        number=1,
        message=data.Debuggee,
    )


class ListActiveBreakpointsRequest(proto.Message):
    r"""Request to list active breakpoints.

    Attributes:
        debuggee_id (str):
            Required. Identifies the debuggee.
        wait_token (str):
            A token that, if specified, blocks the method call until the
            list of active breakpoints has changed, or a server-selected
            timeout has expired. The value should be set from the
            ``next_wait_token`` field in the last response. The initial
            value should be set to ``"init"``.
        success_on_timeout (bool):
            If set to ``true`` (recommended), returns
            ``google.rpc.Code.OK`` status and sets the ``wait_expired``
            response field to ``true`` when the server-selected timeout
            has expired.

            If set to ``false`` (deprecated), returns
            ``google.rpc.Code.ABORTED`` status when the server-selected
            timeout has expired.
    """

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    wait_token = proto.Field(
        proto.STRING,
        number=2,
    )
    success_on_timeout = proto.Field(
        proto.BOOL,
        number=3,
    )


class ListActiveBreakpointsResponse(proto.Message):
    r"""Response for listing active breakpoints.

    Attributes:
        breakpoints (Sequence[google.cloud.debugger_v2.types.Breakpoint]):
            List of all active breakpoints. The fields ``id`` and
            ``location`` are guaranteed to be set on each breakpoint.
        next_wait_token (str):
            A token that can be used in the next method
            call to block until the list of breakpoints
            changes.
        wait_expired (bool):
            If set to ``true``, indicates that there is no change to the
            list of active breakpoints and the server-selected timeout
            has expired. The ``breakpoints`` field would be empty and
            should be ignored.
    """

    breakpoints = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=data.Breakpoint,
    )
    next_wait_token = proto.Field(
        proto.STRING,
        number=2,
    )
    wait_expired = proto.Field(
        proto.BOOL,
        number=3,
    )


class UpdateActiveBreakpointRequest(proto.Message):
    r"""Request to update an active breakpoint.

    Attributes:
        debuggee_id (str):
            Required. Identifies the debuggee being
            debugged.
        breakpoint_ (google.cloud.debugger_v2.types.Breakpoint):
            Required. Updated breakpoint information. The field ``id``
            must be set. The agent must echo all Breakpoint
            specification fields in the update.
    """

    debuggee_id = proto.Field(
        proto.STRING,
        number=1,
    )
    breakpoint_ = proto.Field(
        proto.MESSAGE,
        number=2,
        message=data.Breakpoint,
    )


class UpdateActiveBreakpointResponse(proto.Message):
    r"""Response for updating an active breakpoint.
    The message is defined to allow future extensions.

    """


__all__ = tuple(sorted(__protobuf__.manifest))
